// If this code works, it was written by Svetlio Kapralov. If not, I don't know who wrote it.
package net.playerfinder.jsf.components.rating;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.behavior.ClientBehavior;
import javax.faces.component.behavior.ClientBehaviorContext;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.FacesRenderer;
import javax.faces.render.Renderer;

import net.playerfinder.jsf.components.common.Utils;

@FacesRenderer(componentFamily = UIRating.COMPONENT_FAMILY, rendererType = UIRating.DEFAULT_RENDERER)
public class UIRatingRenderer extends Renderer {

	private static String HIDDEN_INPUT = "_hiddenratingfield";
	private static String RESET_RATING = "0";

	@Override
	public void decode(FacesContext context, UIComponent component) {
		UIRating rating = (UIRating) component;
		if (rating.isDisabled()) {
			return;
		}
		String clientId = rating.getClientId();
		String value = context.getExternalContext().getRequestParameterMap()
				.get(clientId + HIDDEN_INPUT);

		if (value == null || "".equals(value) || RESET_RATING.equals(value))
			rating.setSubmittedValue(new Double(0));
		else
			rating.setSubmittedValue(Double.parseDouble(value));

		Utils.decodeBehaviors(context, component);
	}

	@Override
	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException {
		UIRating rating = (UIRating) component;
		encodeMarkup(context, rating);
	}

	private void encodeMarkup(FacesContext context, UIRating rating)
			throws IOException {
		ResponseWriter writer = context.getResponseWriter();
		String clientId = rating.getClientId(context);

		writer.startElement("span", rating);
		writer.writeAttribute("id", clientId, "id");

		String onchange = null;
		ClientBehaviorContext behaviorContext = ClientBehaviorContext
				.createClientBehaviorContext(context, rating, "change",
						clientId, null);
		Map<String, List<ClientBehavior>> behaviors = rating
				.getClientBehaviors();
		if (behaviors.containsKey("change")) {
			onchange = behaviors.get("change").get(0)
					.getScript(behaviorContext);
		}

		writer.startElement("input", null);
		writer.writeAttribute("type", "text", null);
		writer.writeAttribute("id", clientId + HIDDEN_INPUT, "id");
		writer.writeAttribute("name", clientId + HIDDEN_INPUT, "name");
		Object hiddenInputValue = rating.getValue();
		if (hiddenInputValue != null) {
			writer.writeAttribute("value", hiddenInputValue.toString(), null);
		}
		if (onchange != null)
			writer.writeAttribute("onchange", onchange, null);
		// writer.writeAttribute("style", "display: none;", null);
		writer.writeAttribute("type", "hidden", null);
		writer.endElement("input");

		writer.startElement("div", null);
		String inputId = clientId + "_div";
		writer.writeAttribute("id", inputId, null);
		writer.endElement("div");

		writer.startElement("script", null);
		writer.write("pf_jQuery('#"
				+ Utils.escapeIdForJQuery(clientId + "_div") + "').rateit({ ");
		writer.write("max: " + rating.getTotalStars());
		writer.write(", step: " + rating.getStep());
		writer.write(", readonly: " + rating.isDisabled());
		writer.write(", backingfld: '#"
				+ Utils.escapeIdForJQuery(clientId + HIDDEN_INPUT) + "'");
		writer.write("}); \n");

		writer.write("pf_jQuery('#"
				+ Utils.escapeIdForJQuery(clientId + "_div") + "').bind");
		writer.write("('rated', function (event, value) { \n");
		writer.write("document.getElementById('" + clientId + HIDDEN_INPUT
				+ "').value = value; \n");
		if (onchange != null)
			writer.write("pf_jQuery('#"
					+ Utils.escapeIdForJQuery(clientId + HIDDEN_INPUT)
					+ "').change(); ");
		writer.write("});");

		writer.write("pf_jQuery('#"
				+ Utils.escapeIdForJQuery(clientId + "_div") + "').bind");
		writer.write("('reset', function (event, value) { \n");
		writer.write("document.getElementById('" + clientId + HIDDEN_INPUT
				+ "').value = " + RESET_RATING + "; \n");
		if (onchange != null)
			writer.write("pf_jQuery('#"
					+ Utils.escapeIdForJQuery(clientId + HIDDEN_INPUT)
					+ "').change(); ");
		writer.write("});");
		writer.endElement("script");

		writer.endElement("span");

	}
}
